function [detected_chirplets,lmax]=detectionWinmax(C,nbcr,winmax,pfa);
% [chirplets,sigpoints,lmax,iA]=detectionWinmax(C,nbcr,winmax,pfa);
% A subfunction of chirpgathering.m that iteratively detects significant
% chirplets from the chirplet transform based on a noise level estimated
% using spectral kurtosis [Millioz and Martin 2010] and the given constant
% false alarm rate, pfa.
%
% Input parameters:
%   
%   C           Chirplet transform, indexed in the order f,t,c
%   nbcr        number of chirprates
%   winmax      The upper bound correlation window used for iterative
%               detection (see [Millioz and Davies 2011])
%   pfa          Probability of false alarm for individual T-F detections
%
% Output variables:
%   
%   detected_chirplets  A TF mask of indicating the TF position of 
%                       detected chirplets
%   lmax                A matrix corresponding to the maximum chirplet 
%                       chirprate index at each TF point  
%
%
% A Sub-function of chirpgathering.m
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.


disp('     Detection and windowing')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% noise estimation using the spectral kurtosis of the minimal
% statistics - see Millioz and Martin "Estimation of a white Gaussian noise
% in the Short Time Fourier Transform based on the spectral kurtosis of the
% minimal statistics: Application to underwater noise." ICASSP 2010. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<4
    pfa=1e-3; 
end
kappa=-0.3; % target value for spectral kurtosis
Xn=1.709279; % normalised threoshold

% speed up the computations - only a chirprate close to 0;
D=C(:,:,round((nbcr+1)/2));
valabs=abs(D(:)).^2;
[valabs,ind]=sort(valabs);
Csort=abs(D(ind))';
axi=abs(Csort);

% Estimate the noise variance 
Cvar=cumsum(abs(Csort.^2))./(1:length(Csort));
C4=cumsum(abs(Csort.^4))./(1:length(Csort));
Ckurt=C4./(Cvar.^2) -2 ;

ind=find( abs(Ckurt-kappa) == min(abs(Ckurt-kappa)) );
noisevar=(axi(ind)/Xn)^2;

% Maximal chirplet
[A,lmax]=max(abs(C).^2,[],3);
% half-length of the spectral window
hlw=(length(winmax)-1)/2;
% maximal frequency index
kmax=size(C,1);
detected_chirplets=zeros(size(A));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Iterations to select coefficients %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for indn=1:size(C,2);
  An=A(:,indn);
  A0=A(:,indn);
  while sum(An > (-noisevar*log(1-(1-pfa)^(1/nbcr))));
    % while points are not only noise or selected
    [maxsig,indmax]=max(An);
    detected_chirplets(indmax,indn)=1;
    imin=max(1,indmax-hlw); imax=min(kmax,indmax+hlw);
    iwmin=max(1,hlw+1-indmax+imin); iwmax=min(2*hlw+1,hlw+1-indmax+imax);

    An(imin:imax)=An(imin:imax)-A0(indmax).*winmax(iwmin:iwmax)';
  end
end

% selected chirplets
