function dispsig(sig,fs)
% function dispsig(sig,fs)
% Generate the TF plot for the chirps listed in the sig matrix. Color code
% the chirps according to the signal grouping
%
%   input variables:
%
%   sig         lists all detected chirps in a matrix. Each row identifies
%               a detected chirp and consists of 5 values. The first 4
%               being the same as in the chirps variable in gathering.m.
%               The 5 column indexes the signal to which the chirp is
%               associated:
%
%               chirps(indchirp,:) = ...
%                [start time, initial freq, end time, chirprate, sig index]
%
%               All values are normalised assuming a sampling rate of 1.
%
%  fs           The sampling frequency. Used to reverse the normalisation
%               and generate the correct time and frequency axes in the
%               figure.
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.

% Reverse normalisation
start=sig(:,1); chirp_len=sig(:,3)-sig(:,1); freq=sig(:,2); chirp=sig(:,4);
freq=freq*fs; chirp=chirp*fs*fs; start=start/fs; chirp_len=chirp_len/fs;

% determine the maximum number of signals
nbcolor=max(sig(:,5)); 

% define symbols for marking signals (maximum of 12)
symbols='ox+*sdv^<>ph';


% define colors for coloring signals
map=colormap;
ind=linspace(1,64,nbcolor);
colors=map(round(ind),:);
hold on;

for indtp=1:length(start) % loop through chirps
  if sig(indtp,5) % exclude unassigned chirps (sig(indtp,5)=0)
    color=colors(sig(indtp,5),:);
    symbol=symbols(mod(sig(indtp,5),12));
  else
    color=[1 0 1];
    symbol='.';
  end

  %calculate Chirp TF start and end points
  xs=[start(indtp) start(indtp)+chirp_len(indtp)];
  ys=[freq(indtp) freq(indtp)+chirp(indtp)*chirp_len(indtp)];
  
  % Draw Chirp in TF
  h=line(xs,ys);
  
  % add signal label to plot
  set(h,'Color',color)
  plot(mean(xs),mean(ys),symbol,'Color',color) 
end

axis([0 max(sig(:,3)) 0 fs])
hold off;
