function [chirps,sig_chirps,atoms]=gathering(C,selection,CRmax,nbcr,wlen,t,f,c,lmax,outofbounds);
% [chirps,atoms]=gathering(C,selection,CRmax,nbcr,wlen,t,f,c,lmax,outofbounds);
% Gathers the detected chirplets into consistent linear chirps.
% If outofbounds=0 the chirprates are restricted to be between +/- CRmax.
%
% Input parameters:
%   
%   C               Chirplet transform, indexed in the order f,t,c
%   selection       The TF mask indicating the TF points where a
%                   significant chirplet has been detected
%   CRmax           Chirp Rate Maximum
%   nbcr            number of chirprates
%   wlen            window length (longer windows are prefered)
%   t               the time values for the time index of C
%   f               the freq values for the freq index of C
%   c               the chirprate values for the chirprate index of C
%   lmax            The chirprate index for the maximum chirplet at each TF
%                   point
%   outofbouinds    if = 1 look for chirps beyond +/- CRmax
%
% Output variables:
%
%   chirps          lists all detected chirps in a matrix. Each row
%                   identifies a detected chirp and consists of 4 values:
%
%                   chirps(indchirp,:) = ...
%                      [start time, initial frequency, end time, chirprate] 
%
%   sig_chirps      lists only chirps that that are of a reasonable length
%                   - all chirps of length less than or equal to deltat are
%                   removed
%
%   atoms           A TF matrix indicating to which chirp in the all chirps
%                   list the chirplet at that TF point is associated.
%
% A Sub-function of chirpgathering.m
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.

if outofbounds
 disp('search for all chirprates')
end
debug=0;
Lh=(wlen-1)/2;
chirps=[];            % chirps
usefulchirps=[];      % flag for chirps
atoms=zeros(size(C,1),size(C,2)); % association t/f <=> chirp number

dt=t(2)-t(1); df=f(2)-f(1); 

% Parameters for grouping chirplets into chirps 
% (see [Millioz and Davies 2012] for details)
deltat=3*dt;          % to extend the chirp
deltac=1.5*CRmax/(nbcr-1);
deltaf=max(df,deltac*deltat/2);

indices=find(selection); % index selection vectorwise

% first chirplet is the first chirp - calculate the time index (indn), the
% frequency index (indk) and the chirprate index (indc) of the first
% detected chirplet.
tmp=indices(1)-1;
indn=(tmp-mod(tmp,length(f)))/length(f) +1;
indk=tmp - (indn-1)*length(f) +1;
indc=lmax(tmp+1); % index of the best chirprate at (t,f) point

% atom's parameters
at=t(indn);   % time
af=f(indk);   % frequency
ac=c(indc);   % chirprate
% add new chirp to the chirp list
chirps(end+1,:)=[at-Lh af-ac*Lh at+Lh ac];
atoms(indk,indn)=1; % this is the first chirp!

for indtmp=2:length(indices); % on all selected chirplets
    % t-f-c indexes of detected chirplet
    ind=indices(indtmp);
    tmp=ind-1;
    indn=(tmp-mod(tmp,length(f)))/length(f) +1;
    indk=tmp - (indn-1)*length(f) +1;
    indc=lmax(ind); % index of the betst chirprate at (t,f) point
    
    % new atom's t-f-c indices
    at=t(indn);   % time
    af=f(indk);   % frequency
    ac=c(indc);   % chirprate
    
    % chirps satisfying time proximity to new atom (logical vector)
    timecondition = (at>=chirps(:,1)).*(at<=(chirps(:,3)+deltat)); 

    % chirps satisfying frequency proximity to new atom
    frequencycondition = abs((at-chirps(:,1)).*chirps(:,4)+chirps(:,2)-af)<=deltaf;
    
    % More relaxed grouping conditions if outofbounds=1 (see paper)
    if outofbounds % search for chirps with chirprates outside +/1 CRmax
        highchirp = (chirps(:,4)==CRmax).* (chirps(:,3)-chirps(:,1) == 2*Lh) .* (af > (at-chirps(:,1)).*chirps(:,4)+chirps(:,2));
        lowchirp = (chirps(:,4)==-CRmax).* (chirps(:,3)-chirps(:,1) == 2*Lh) .* (af < (at-chirps(:,1)).*chirps(:,4)+chirps(:,2));
        stricttimecondition = (at>=chirps(:,1)+dt).*(at<=(chirps(:,3)+dt));
        frequencycondition = frequencycondition + (highchirp + lowchirp).*stricttimecondition;
    end    
    
    % chirps satisfying the chirprate proximity to new atom
    chirpscondition = abs(ac-max(-CRmax,min(CRmax,chirps(:,4))))<=deltac;
    
    % logical and...
    matchingchirp = timecondition.*frequencycondition.*chirpscondition;
    
    if ~sum(matchingchirp) % create new chirp
        chirps(end+1,:)=[at-Lh af-ac*Lh at+Lh ac];
        atoms(indk,indn)=size(chirps,1);
    else 
        if sum(matchingchirp)>1; % take the closest from the chirp's frequency line
            freqline=abs((at-chirps(:,1)).*chirps(:,4)+chirps(:,2)-af);
            indchirp=find( abs(freqline./matchingchirp)  == min(abs(freqline./matchingchirp)) );
            if length(indchirp)>1   % bad luck
                indchirp=indchirp(1); % why not?
                disp('Warning: ambiguity in chirplet assignation'); % shouldn't appear
            end
        else   % allocate to single associated chirp
            indchirp=find(matchingchirp);
        end
        atoms(indk,indn)=indchirp;

        % update the molecule
        inda=find(atoms==indchirp); % all chirplets associated with the same chirp
        tmp=inda-1;
        indan=(tmp-mod(tmp,length(f)))/length(f) +1;
        indak=tmp - (indan-1)*length(f) +1;
        indac=lmax(inda);
        times=t(indan);      freqs=f(indak);      crs=c(indac);
        
        % recalculate chirp parameters: start time, initial frequency, end time, chirprate
        
        if (length(times)>2)  & (max(times) > min(times))
            % 3 or more points: robust linear regression for chirprate/frequency
            freqchirptmp=robustfit(times,freqs-mean(freqs));
            freqchirptmp(1)=freqchirptmp(1)+mean(freqs);
            chirps(indchirp,:)=[min(times)-Lh freqchirptmp(1)+freqchirptmp(2)*(min(times)-Lh) max(times)+Lh freqchirptmp(2)];
            
        elseif (length(times)==2) & times(1)~=times(2)
            % 2 different time points: interpolate
            chirpratetmp=(freqs(2)-freqs(1))/(times(2)-times(1));
            chirps(indchirp,:)=[times(1)-Lh freqs(1)-Lh*chirpratetmp times(2)+Lh chirpratetmp];
            
        else %length(times)=2 and times(1)=times(2): stupid average otherwise (which should not happen if signal detection is correct)
            chirps(indchirp,:)=[times(1)-Lh mean(freqs)-Lh*mean(crs) times(1)+Lh mean(crs)];
        end
        %atinchirp=1;
        atoms(indk,indn)=indchirp;
    end
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Post-processing -remove very short chirps                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Keep only chirps of length >= 4*dt
sig_chirps=chirps(chirps(:,3)-chirps(:,1)>deltat,:); 

