function [winmax,allwin] = maxwindow(wlen,nbfreq,CRmax,nbcr,typef);
% A subfunction of chirpgathering.m
%
% This function calculates the upper bound window for the iterative masking
% chirplet detection as set out in Millioz and Davies, "Sparse detection in
% the chirplet transform: application to FMCW radar signals," IEEE Trans.
% SP, 2013.  
% 
% Input parameters:
%   
%   wlen          window length (longer windows are prefered)
%   nbfreq        number of frequencies
%   CRmax         Chirp Rate Maximum
%   nbcr          number of chirprates
%   typef         window type used in analysis:
%                'r' = 'Rect'
%                'k' = 'Kaiser',4
%                'h' = 'Hanning'
%                'hm' = 'Hamming'
%                'b' = 'Blackman'
%                'g' = 'Gauss' [DEFAULT]
%
% Output variables:
%
%   winmax      the upper bound window
%   allwin      the collection of windowing functions for each chirprate 
%
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publication that makes use of this
% software. Copyright 2016, University of Edinburgh.

switch typef
    case 'r'
  win=tftb_window(wlen,'Rect');
    case 'k'
  win=tftb_window(wlen,'Kaiser',4);
    case 'h'
  win=tftb_window(wlen,'Hanning');
    case 'hm'
  win=tftb_window(wlen,'Hamming');
    case 'b'
  win=tftb_window(wlen,'Blackman');
    case 'g'
  win=tftb_winow(wlen,'Gauss');
end
clim=1/sum(win)^2;
deltac=2*CRmax/(nbcr-1);

CRmax=2*CRmax;

kmax=min(nbfreq/2-1,round(CRmax*wlen/2*nbfreq)); % half maximal size

if deltac<clim % at least a chirplet coefficient has maximum energy
  win0=abs(fft(win,nbfreq));
  win0shift=abs(fft(win'.*exp(2*pi*j*0.5/nbfreq*[1:wlen]),nbfreq));

  kmin=ceil(clim*wlen*nbfreq/2); % half minimal size
  allwin=zeros(1+kmax-kmin,kmax+1);
  allwin(end,:)=win0(1:kmax+1);
else
  kmin=ceil(deltac/2*wlen/2*nbfreq);  % half minimal size
  allwin=zeros(kmax-kmin,kmax+1);
end

for ind=kmin:kmax
  c=min(CRmax,(2*ind)/(wlen*nbfreq));
  switch typef
        case 'r'
      winc=1/sqrt(c)*tftb_window(2*ind,'Rect');
        case 'k'
      winc=1/sqrt(c)*tftb_window(2*ind,'Kaiser',4);
        case 'h'
      winc=1/sqrt(c)*tftb_window(2*ind,'Hanning');
        case 'hm'
      winc=1/sqrt(c)*tftb_window(2*ind,'Hamming');
        case 'b'
      winc=1/sqrt(c)*tftb_window(2*ind,'Blackman');
        case 'g'
      winc=1/sqrt(c)*tftb_winow(2*ind,'Gauss');
  end
  allwin(ind-kmin+1,1:ind+1)=winc(ind:end);
end
halfwinmax=max(allwin);
winmax(1:kmax+1)=halfwinmax(end:-1:1);
winmax(kmax+1:2*kmax+1)=halfwinmax;
