function [signals]=signal_gathering(sig_chirp,t,f,CRmax)
% [signals]=signal_gathering(chirpsf,t,f,CRmax);
% gathers individual chirps into continuous wave piecewise linear signals.
%
% Input parameters:
%   
%   sig_chirps      lists significant detected chirps created in
%                   chirpgathering.m in a matrix. Each row
%                   identifies a detected chirp and consists of 4 values:
%
%                   chirps(indchirp,:) = ...
%                      [start time, initial frequency, end time, chirprate] 
%
%   t               the time values for the time indices of the chirplet
%                   transform that generated the sig_chirps list  
%   f               the freq values for the freq indices of the chirplet
%                   transform that generated the sig_chirps list 
%   CRmax           Chirp Rate Maximum
%
% Output variables:
%
%   signals         lists all detected chirps in a matrix. Each row
%                   identifies a detected chirp and consists of 4 values:
%
%                   chirps(indchirp,:) = ...
%                      [start time, initial frequency, end time, chirprate] 
%
% Code written by Fabien Millioz and Mike Davies, last update 2016. This
% function is part of a software implementation of the Maximum Chirplet
% Transform and its application to the detection of piecewise linear chirp
% signals. Theory and details of the technique can be found in:
%
%   Millioz and Davies, "Sparse detection in the chirplet transform:
%   application to FMCW radar signals," IEEE Transactions on Sig. Proc. 
%   vol 60(6), pp 2800  2813, 2012.
%
% Please reference this paper in any publicatiosn that makes use of this
% software. Copyright 2016.


% define chirp tolerance as in gathering.m
dt=t(2)-t(1);
deltat=3*dt;          % to extend the chirp

l=length(sig_chirp);
timebegs=sig_chirp(:,1)*ones(1,l); % list of chirp time starts
freqbegs=sig_chirp(:,2)*ones(1,l); % list of chirp frequency starts
timeends=ones(l,1)*sig_chirp(:,3)'; % list of chirp time ends
freqends=ones(l,1)* (sig_chirp(:,2)+sig_chirp(:,4).*(sig_chirp(:,3)-sig_chirp(:,1)))'; % list of chirp frequency ends

% normalization
timebegs=timebegs/max(t);
timeends=timeends/max(t);
freqbegs=freqbegs/max(f);
freqends=freqends/max(f);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check start and end of signals (note we should allow some leaway here -
% such as the number of chirps starting/ending within deltat of the
% start/end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% number of starting chirps
startchirps=sum(sig_chirp(:,1)<=deltat+1);
disp(['Number of signals detected at the start of the data: ',num2str(startchirps)]);

% number of finishing chirps
finchirps=sum(sig_chirp(:,3)>=max(sig_chirp(:,3))-deltat);
disp(['Number of signals detected at the end of the data: ',num2str(finchirps)]);

% should be the same...
if startchirps~=finchirps
    disp('WARNING: different number of signal at the beginning and at the end');
end


% Calculate pairwise distances between chirp ends and other chirp starts
dist=sqrt( (timeends-timebegs).^2 + (freqends-freqbegs).^2); % pairwise distances

% avoid self assignment - artificially increase diagonal distances
dist=dist+max(max(dist)).*eye(size(dist));

% remove the ending chirps 
diste=dist(:,find(sig_chirp(:,3)<max(sig_chirp(:,3))-deltat));

% mask the beginning chirps (assuming that these are listed first as in
% gathering.m)
diste(1:startchirps,:)=max(max(diste))+1;

% associating minimum distance to each chirp end and identify links
[minval,links]=min(diste);

% keep only if they are close in the TF, accounting for TF uncertainty (see
% eqn (52) [Millioz and Davies])
threshold=dt*sqrt(max(t)^-2+(CRmax)^2); %
close=minval<threshold;

% check that there is no double at this point (there is probably)
ll=length(links);
checklink=(ones(ll,1)*links)==(links'*ones(1,ll))-eye(ll);

if sum(sum(checklink))~=0 % if not, we remove them
    disp('warning: convergence of chirps, keeping the close in TF, switch to time criterion.');
    diste(diste>threshold)=max(max(diste));
    [minval,links]=min(diste);
    links(links==1)=0; % when all values are equal, the first one is selected: no answer
end

dist2=abs(timeends-timebegs);
dist2([1:startchirps links(links~=0)],:)=max(max(dist2));
[minval2,links2]=min(dist2);
links(links==0)=links2(links==0);

% final check (...)
checklink=(ones(ll,1)*links)==(links'*ones(1,ll))-eye(ll);
if sum(sum(checklink))~=0 % if not, we remove them
    disp('warning: convergence of chirps, again and again: give up');
end

% assign a signal number to the chirps
signals=zeros(l,5);
signals(:,1:4)=sig_chirp;
indstart=find(sig_chirp(:,1)<=deltat+1);
translate=zeros(1,length(sig_chirp));
indtmp=find(sig_chirp(:,3)<max(sig_chirp(:,3))-deltat);
translate(indtmp)=links;
links=translate;

prec1=0; prec2=0; % avoid infinite loop
for ind=1:startchirps;
    curind=indstart(ind);
    signals(curind,5)=ind;
    prec1=curind;
    while sig_chirp(curind,3)<max(sig_chirp(:,3))-deltat
        prec2=prec1;
        prec1=curind;
        curind=links(curind);
        signals(curind,5)=ind;
        if prec2==curind
            break
        end
    end
end
